/*
	Copyright (C) MINZKN.COM
	All rights reserved.
	Author: JAEHYUK CHO <mailto:minzkn@minzkn.com>
*/

/* ---- */

#include <stdio.h>
#include <stdlib.h>

#include <net/if.h> /* for IFF_XXX */
#if 0L
# include <net/if_arp.h> /* for ARPHRD_XXX */
#endif

#include <arpa/inet.h> /* for inet_ntop */

#if __GLIBC_PREREQ(2,3)
# include <ifaddrs.h> /* >= glibc 2.3 */
# include <netpacket/packet.h> /* for struct sockaddr_ll */

int main(int s_argc, char **s_argv)
{
	struct ifaddrs *s_ifaddrs;
	const struct ifaddrs *s_ifaddrs_trace;
	int s_check;

	char s_address_string[ 128 ];

	struct sockaddr_table_ts {
		const char *m_type_name;
		const struct sockaddr *m_sockaddr;
	}s_table[ 4 ];
	int s_table_index;

	(void)s_argc;
	(void)s_argv;

	s_ifaddrs = NULL;
	s_check = getifaddrs((struct ifaddrs **)(&s_ifaddrs));
	if(s_check == (-1)) {
		perror("getifaddrs");
		return(EXIT_FAILURE);
	}

	for(s_ifaddrs_trace = s_ifaddrs;s_ifaddrs_trace != NULL;s_ifaddrs_trace = s_ifaddrs_trace->ifa_next) {
		s_table[0].m_type_name = "address";
		s_table[0].m_sockaddr = (const struct sockaddr *)s_ifaddrs_trace->ifa_addr;
		
		s_table[1].m_type_name = "netmask";
		s_table[1].m_sockaddr = (const struct sockaddr *)s_ifaddrs_trace->ifa_netmask;
		
		if((s_ifaddrs_trace->ifa_flags & IFF_BROADCAST) == IFF_BROADCAST) {
			s_table[2].m_type_name = "broadaddr";
			s_table[2].m_sockaddr = (const struct sockaddr *)s_ifaddrs_trace->ifa_broadaddr;
		}
		else if((s_ifaddrs_trace->ifa_flags & IFF_POINTOPOINT) == IFF_POINTOPOINT) {
			s_table[2].m_type_name = "dstaddr";
			s_table[2].m_sockaddr = (const struct sockaddr *)s_ifaddrs_trace->ifa_dstaddr;
		}
		else {
			s_table[2].m_type_name = NULL;
			s_table[2].m_sockaddr = NULL;
		}
			
		s_table[3].m_type_name = NULL;
		s_table[3].m_sockaddr = NULL;

		(void)fprintf(
			stdout,
			"%s: flags=0x%X{IFF_XXXX:\"net/if.h\",\"linux/if.h\"}\n",
			s_ifaddrs_trace->ifa_name ? : "<NULL>",
			s_ifaddrs_trace->ifa_flags
		);

		for(s_table_index = 0;s_table[s_table_index].m_sockaddr != NULL;s_table_index++) {
			switch(s_table[s_table_index].m_sockaddr->sa_family) {
				case AF_INET: /* struct sockaddr_in */ {
					const struct sockaddr_in *s_sockaddr_in_ptr;

					s_sockaddr_in_ptr = (const struct sockaddr_in *)s_table[s_table_index].m_sockaddr;

					(void)fprintf(
						stdout,
						"\t- family=%s=%d, %s=\"%s\", port=%u\n",
						"AF_INET",
						s_table[s_table_index].m_sockaddr->sa_family,
						s_table[s_table_index].m_type_name,
						inet_ntop(
							s_sockaddr_in_ptr->sin_family,
							(const void *)(&s_sockaddr_in_ptr->sin_addr),
							(char *)(&s_address_string[0]),
							(socklen_t)sizeof(s_address_string)
						) ? : "<UNKNOWN>",
						(unsigned int)ntohs(s_sockaddr_in_ptr->sin_port)
					);
													   }
					break;
				case AF_INET6: /* struct sockaddr_in6 */ {
					const struct sockaddr_in6 *s_sockaddr_in6_ptr;

					s_sockaddr_in6_ptr = (const struct sockaddr_in6 *)s_table[s_table_index].m_sockaddr;

					(void)fprintf(
						stdout,
						"\t- family=%s=%d, %s=\"%s\", port=%u, flowinfo=%u, scope_id=%u\n",
						"AF_INET6",
						s_table[s_table_index].m_sockaddr->sa_family,
						s_table[s_table_index].m_type_name,
						inet_ntop(
							s_sockaddr_in6_ptr->sin6_family,
							(const void *)(&s_sockaddr_in6_ptr->sin6_addr),
							(char *)(&s_address_string[0]),
							(socklen_t)sizeof(s_address_string)
						) ? : "<UNKNOWN>",
						(unsigned int)ntohs(s_sockaddr_in6_ptr->sin6_port),
						(unsigned int)s_sockaddr_in6_ptr->sin6_flowinfo,
						(unsigned int)s_sockaddr_in6_ptr->sin6_scope_id
					);
														 }
					break;
				case AF_PACKET: /* struct sockaddr_ll - linux 에서는 AF_PACKET이지만 BSD에서는 AF_LINK(struct sockaddr_sdl) 형태가 있음 */ {
					const struct sockaddr_ll *s_sockaddr_ll;
					size_t s_from_offset;
					size_t s_to_offset;

					s_sockaddr_ll = (const struct sockaddr_ll *)s_table[s_table_index].m_sockaddr;

					for(s_from_offset = (size_t)0u, s_to_offset = (size_t)0u;s_from_offset < ((size_t)s_sockaddr_ll->sll_halen);s_from_offset++) {
						s_to_offset += (size_t)snprintf(
							(char *)(&s_address_string[s_to_offset]),
							sizeof(s_address_string) - s_to_offset,
							"%s%02X",
							(s_from_offset == ((size_t)0u)) ? "" : ":",
							(unsigned int)s_sockaddr_ll->sll_addr[s_from_offset]
						);
					}
					s_address_string[s_to_offset] = '\0';

					(void)fprintf(
						stdout,
						"\t- family=AF_PACKET=%d, protocol=%u, ifindex=%d, hatype=%u{ARPHRD_XXX:\"net/if_arp.h\",\"linux/if_arp.h\"}, pkttype=%u, halen=%u, %s=\"%s\"\n",
						s_table[s_table_index].m_sockaddr->sa_family,
						(unsigned int)s_sockaddr_ll->sll_protocol,
						(int)s_sockaddr_ll->sll_ifindex,
						(unsigned int)s_sockaddr_ll->sll_hatype, /* ARPHRD_LOOPBACK, ARPHRD_ETHER, ... (SeeAlso : "linux/if_arp.h") */ 
						(unsigned int)s_sockaddr_ll->sll_pkttype,
						(unsigned int)s_sockaddr_ll->sll_halen,
						s_table[s_table_index].m_type_name,
						(char *)(&s_address_string[0])
					);

								}
					break;
				default:
					(void)fprintf(
						stdout,
						"\t- family=AF_XXX=%d, %s=?\n",
						s_table[s_table_index].m_sockaddr->sa_family,
						s_table[s_table_index].m_type_name
					);
					break;
			}
		}

		if((s_ifaddrs_trace->ifa_addr != NULL) && (s_ifaddrs_trace->ifa_addr->sa_family == AF_PACKET) && (s_ifaddrs_trace->ifa_data != NULL)) {
#if 0L /* 64-bits structure : https://gist.github.com/beejeebus/cc4fb07472cf5a0afd41 */
			/* TODO */
#else /* 32-bits structure : by getifaddrs() */
# include <linux/if_link.h> /* for struct rtnl_link_stats */
			const struct rtnl_link_stats *s_link_stats;

			s_link_stats = (const struct rtnl_link_stats *)s_ifaddrs_trace->ifa_data;
			(void)fprintf(
				stdout,
				"\t- rx_packets=%llu, rx_bytes=%llu, rx_errors=%llu, rx_dropped=%llu\n"
				"\t- tx_packets=%llu, tx_bytes=%llu, tx_errors=%llu, tx_dropped=%llu\n"
				"\t- multicast=%llu, collisions=%llu\n"
				"\t- rx_length_errors=%llu, rx_over_errors=%llu, rx_crc_errors=%llu, rx_frame_errors=%llu, rx_fifo_errors=%llu, rx_missed_errors=%llu\n"
				"\t- tx_aborted_erros=%llu, tx_carrier_errors=%llu, tx_fifo_errors=%llu, tx_heartbeat_errors=%llu, tx_window_errors=%llu\n"
				"\t- rx_compressed=%llu, tx_compressed=%llu\n",
				(unsigned long long)s_link_stats->rx_packets, /* total packets received   */
				(unsigned long long)s_link_stats->rx_bytes, /* total bytes received     */
				(unsigned long long)s_link_stats->rx_errors, /* bad packets received     */
				(unsigned long long)s_link_stats->rx_dropped, /* no space in linux buffers    */
				/* - */
				(unsigned long long)s_link_stats->tx_packets, /* total packets transmitted    */
				(unsigned long long)s_link_stats->tx_bytes, /* total bytes transmitted  */
				(unsigned long long)s_link_stats->tx_errors, /* packet transmit problems */
				(unsigned long long)s_link_stats->tx_dropped, /* no space available in linux  */
				/* - */
				(unsigned long long)s_link_stats->multicast, /* multicast packets received   */
				(unsigned long long)s_link_stats->collisions,
				/* - */
				(unsigned long long)s_link_stats->rx_length_errors,
				(unsigned long long)s_link_stats->rx_over_errors, /* receiver ring buff overflow  */
				(unsigned long long)s_link_stats->rx_crc_errors, /* recved pkt with crc error    */
				(unsigned long long)s_link_stats->rx_frame_errors, /* recv'd frame alignment error */
				(unsigned long long)s_link_stats->rx_fifo_errors, /* recv'r fifo overrun      */
				(unsigned long long)s_link_stats->rx_missed_errors, /* receiver missed packet   */
				/* - */
				(unsigned long long)s_link_stats->tx_aborted_errors,
				(unsigned long long)s_link_stats->tx_carrier_errors,
				(unsigned long long)s_link_stats->tx_fifo_errors,
				(unsigned long long)s_link_stats->tx_heartbeat_errors,
				(unsigned long long)s_link_stats->tx_window_errors,
				/* - */
				(unsigned long long)s_link_stats->rx_compressed,
				(unsigned long long)s_link_stats->tx_compressed
			);
#endif
		}
	}

	if(s_ifaddrs != NULL) {
		freeifaddrs(s_ifaddrs);
	}

	return(EXIT_SUCCESS);
}
#else
int main(int s_argc, char **s_argv)
{
	(void)s_argc;
	(void)s_argv;
	(void)fprintf(stderr, "not supported getifaddrs !\n");
	return(EXIT_FAILURE);
}
#endif

/* vim:set noexpandtab tabstop=4 shiftwidth=4 softtabstop=4 autoindent cindent smarttab fileencoding=utf8: */
/* End of source */
